// If this code works, it was written by Svetlio Kapralov. If not, I don't know who wrote it.
package net.playerfinder.jsf.components.inputtextarea;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.ConverterException;
import javax.faces.render.FacesRenderer;

import net.playerfinder.jsf.components.common.Utils;

import com.sun.faces.renderkit.Attribute;
import com.sun.faces.renderkit.AttributeManager;
import com.sun.faces.renderkit.RenderKitUtils;
import com.sun.faces.renderkit.html_basic.TextareaRenderer;

@FacesRenderer(componentFamily = UIInputTextArea.COMPONENT_FAMILY, rendererType = UIInputTextArea.DEFAULT_RENDERER)
public class UIInputTextAreaRenderer extends TextareaRenderer {

	private static final Attribute[] ATTRIBUTES = AttributeManager
			.getAttributes(AttributeManager.Key.INPUTTEXTAREA);

	private void encodeScript(FacesContext context,
			UIInputTextArea inputTextarea) throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		String clientId = inputTextarea.getClientId(context);

		String jquerySelector = "pf_jQuery('textarea[name="
				+ Utils.escapeIdForJQuery(clientId) + "]')";

		writer.startElement("script", null);

		String animate = "";
		if (inputTextarea.isAnimate())
			animate = "animate : " + inputTextarea.getAnimateDuration()
					+ ", \n";
		else
			animate = "animate : false, \n";
		String minHeight = "";
		if (inputTextarea.getMinHeight() > 0)
			minHeight = "minHeight: " + inputTextarea.getMinHeight() + ", \n";
		String maxHeight = "";
		if (inputTextarea.getMaxHeight() > 0)
			maxHeight = "maxHeight: " + inputTextarea.getMaxHeight() + ", \n";

		// auto resize
		writer.write("pf_jQuery(function() { " + jquerySelector + ".autoResize({ "
				+ minHeight + maxHeight + animate + "extraSpace : "
				+ inputTextarea.getExtraSpace() + "});");

		// prompt text
		writer.write(jquerySelector + ".prompt({ text: '"
				+ inputTextarea.getPromptText() + "'," + "color: '"
				+ inputTextarea.getPromptColor() + "'});");

		writer.write("});");
		writer.endElement("script");

	}

	@Override
	protected void getEndTextToRender(FacesContext context,
			UIComponent component, String currentValue) throws IOException {

		ResponseWriter writer = context.getResponseWriter();
		assert (writer != null);

		String styleClass = (String) component.getAttributes()
				.get("styleClass");

		UIInputTextArea textarea = (UIInputTextArea) component;
		writer.startElement("span", null);
		String placeholder = textarea.getClientId() + "_span_placeholder";
		writer.writeAttribute("id", placeholder, null);
		writer.writeAttribute("name", placeholder, null);

		writer.startElement("textarea", component);
		writeIdAttributeIfNecessary(context, writer, component);
		writer.writeAttribute("name", component.getClientId(context),
				"clientId");
		if (null != styleClass) {
			writer.writeAttribute("class", styleClass, "styleClass");
		}

		// style is rendered as a passthru attribute
		RenderKitUtils.renderPassThruAttributes(context, writer, component,
				ATTRIBUTES);
		RenderKitUtils.renderXHTMLStyleBooleanAttributes(writer, component);

		RenderKitUtils.renderOnchange(context, component, false);

		// render default text specified
		if (currentValue != null) {
			writer.writeText(currentValue, component, "value");
		}

		writer.endElement("textarea");

		encodeScript(context, textarea);

		writer.endElement("span");

	}

	@Override
	public Object getConvertedValue(FacesContext context,
			UIComponent component, Object submittedValue)
			throws ConverterException {
		String text = (String) submittedValue;
		UIInputTextArea textArea = (UIInputTextArea) component;
		// if nothing is written then text area gets the prompt text value. In
		// this case we return null instead.
		if (textArea.getPromptText() != null && text != null
				&& text.equals(textArea.getPromptText())) {
			return null;
		}
		return super.getConvertedValue(context, component, submittedValue);
	}
}
